@@ -25,22 +25,22 @@ int VL_GetEntryBYIDN(struct rx_connection *, afs_int32, afs_int32, struct nvldbe
 int VL_GetEntryByNameN(struct rx_connection *, char *, struct nvldbentry *);
 
 /* from .xg file */
-extern StartRXAFS_FetchData (struct rx_call *,
+int StartRXAFS_FetchData (struct rx_call *,
 	struct AFSFid *Fid,
 	afs_int32 Pos, 
 	afs_int32 Length);
-extern EndRXAFS_FetchData (struct rx_call *,
+int EndRXAFS_FetchData (struct rx_call *,
 	struct AFSFetchStatus *OutStatus, 
 	struct AFSCallBack *CallBack, 
 	struct AFSVolSync *Sync);
 
-extern RXAFS_FetchACL(struct rx_connection *,
+int RXAFS_FetchACL(struct rx_connection *,
 	struct AFSFid *Fid, 
 	struct AFSOpaque *AccessList, 
 	struct AFSFetchStatus *OutStatus, 
 	struct AFSVolSync *Sync);
 
-extern RXAFS_FetchStatus (struct rx_connection *,
+int RXAFS_FetchStatus (struct rx_connection *,
 	struct AFSFid *Fid, 
 	struct AFSFetchStatus *OutStatus, 
 	struct AFSCallBack *CallBack, 
@@ -57,6 +57,44 @@ int EndRXAFS_StoreData(struct rx_call *,
 	struct AFSFetchStatus *OutStatus, 
 	struct AFSVolSync *Sync);
 
+int StartRXAFS_FetchData64(struct rx_call *z_call,
+	struct AFSFid * Fid,
+	afs_int64 Pos,
+	afs_int64 Length);
+
+int EndRXAFS_FetchData64(struct rx_call *z_call,
+        struct AFSFetchStatus * OutStatus,
+	struct AFSCallBack * CallBack,
+	struct AFSVolSync * Sync);
+
+afs_int32 SRXAFS_FetchData64(struct rx_call *z_call,
+	struct AFSFid * Fid,
+	afs_int64 Pos,
+	afs_int64 Length,
+	struct AFSFetchStatus * OutStatus,
+	struct AFSCallBack * CallBack,
+	struct AFSVolSync * Sync);
+
+int StartRXAFS_StoreData64(struct rx_call *z_call,
+	struct AFSFid * Fid,
+	struct AFSStoreStatus * InStatus,
+	afs_uint64 Pos,
+	afs_uint64 Length,
+	afs_uint64 FileLength);
+
+int EndRXAFS_StoreData64(struct rx_call *z_call,
+	struct AFSFetchStatus * OutStatus,
+	struct AFSVolSync * Sync);
+
+afs_int32 SRXAFS_StoreData64(struct rx_call *z_call,
+	struct AFSFid * Fid,
+	struct AFSStoreStatus * InStatus,
+	afs_uint64 Pos,
+	afs_uint64 Length,
+	afs_uint64 FileLength,
+	struct AFSFetchStatus * OutStatus,
+	struct AFSVolSync * Sync);
+
 int RXAFS_StoreACL (struct rx_connection *,
 	struct AFSFid *Fid, 
 	struct AFSOpaque *AccessList,  
@@ -193,6 +231,8 @@ int RXAFS_Lookup (struct rx_connection *,
 	struct AFSCallBack *CallBack,
 	struct AFSVolSync *Sync);
 
+#define CM_DEFAULT_CALLBACKPORT         7001
+
 /* common flags to many procedures */
 #define CM_FLAG_CREATE		1		/* create entry */
 #define CM_FLAG_CASEFOLD	2		/* fold case in namei, lookup, etc. */
@@ -219,7 +259,7 @@ int RXAFS_Lookup (struct rx_connection *,
 #define CM_ERROR_EXISTS			(CM_ERROR_BASE+11)
 #define CM_ERROR_CROSSDEVLINK		(CM_ERROR_BASE+12)
 #define CM_ERROR_BADOP			(CM_ERROR_BASE+13)
-/* CM_ERROR_BADPASSWORD used to be here */
+#define CM_ERROR_BADPASSWORD            (CM_ERROR_BASE+14)
 #define CM_ERROR_NOTDIR			(CM_ERROR_BASE+15)
 #define CM_ERROR_ISDIR			(CM_ERROR_BASE+16)
 #define CM_ERROR_READONLY		(CM_ERROR_BASE+17)
@@ -245,5 +285,15 @@ int RXAFS_Lookup (struct rx_connection *,
 #define CM_ERROR_BUFFERTOOSMALL		(CM_ERROR_BASE+38)
 #define CM_ERROR_RENAME_IDENTICAL	(CM_ERROR_BASE+39)
 #define CM_ERROR_ALLOFFLINE             (CM_ERROR_BASE+40)
-#define CM_ERROR_AMBIGUOUS_FILENAME (CM_ERROR_BASE+41)
+#define CM_ERROR_AMBIGUOUS_FILENAME     (CM_ERROR_BASE+41)
+#define CM_ERROR_BADLOGONTYPE	        (CM_ERROR_BASE+42)
+#define CM_ERROR_GSSCONTINUE            (CM_ERROR_BASE+43)
+#define CM_ERROR_TIDIPC                 (CM_ERROR_BASE+44)
+#define CM_ERROR_TOO_MANY_SYMLINKS      (CM_ERROR_BASE+45)
+#define CM_ERROR_PATH_NOT_COVERED       (CM_ERROR_BASE+46)
+#define CM_ERROR_LOCK_CONFLICT          (CM_ERROR_BASE+47)
+#define CM_ERROR_SHARING_VIOLATION      (CM_ERROR_BASE+48)
+#define CM_ERROR_ALLDOWN                (CM_ERROR_BASE+49)
+#define CM_ERROR_TOOFEWBUFS		(CM_ERROR_BASE+50)
+#define CM_ERROR_TOOMANYBUFS		(CM_ERROR_BASE+51)
 #endif /*  __CM_H_ENV__ */
