@@ -68,7 +68,7 @@ static void CleanupACLEnt(cm_aclent_t * aclp)
  * Get an acl cache entry for a particular user and file, or return that it doesn't exist.
  * Called with the scp locked.
  */
-long cm_FindACLCache(cm_scache_t *scp, cm_user_t *userp, long *rightsp)
+long cm_FindACLCache(cm_scache_t *scp, cm_user_t *userp, afs_uint32 *rightsp)
 {
     cm_aclent_t *aclp;
     long retval = -1;
@@ -91,14 +91,16 @@ long cm_FindACLCache(cm_scache_t *scp, cm_user_t *userp, long *rightsp)
                            &aclp->q);
             } else {
                 *rightsp = aclp->randomAccess;
-                if (cm_data.aclLRUEndp == aclp)
-                    cm_data.aclLRUEndp = (cm_aclent_t *) osi_QPrev(&aclp->q);
+		if (cm_data.aclLRUp != aclp) {
+		    if (cm_data.aclLRUEndp == aclp)
+			cm_data.aclLRUEndp = (cm_aclent_t *) osi_QPrev(&aclp->q);
 
-                /* move to the head of the LRU queue */
-                osi_QRemove((osi_queue_t **) &cm_data.aclLRUp, &aclp->q);
-                osi_QAddH((osi_queue_t **) &cm_data.aclLRUp,
-                           (osi_queue_t **) &cm_data.aclLRUEndp,
-                           &aclp->q);
+		    /* move to the head of the LRU queue */
+		    osi_QRemove((osi_queue_t **) &cm_data.aclLRUp, &aclp->q);
+		    osi_QAddH((osi_queue_t **) &cm_data.aclLRUp,
+			       (osi_queue_t **) &cm_data.aclLRUEndp,
+			       &aclp->q);
+		}
                 retval = 0;     /* success */
             }               
             break;
@@ -145,7 +147,7 @@ static cm_aclent_t *GetFreeACLEnt(cm_scache_t * scp)
  *
  * The scp must be locked when this function is called.
  */
-long cm_AddACLCache(cm_scache_t *scp, cm_user_t *userp, long rights)
+long cm_AddACLCache(cm_scache_t *scp, cm_user_t *userp, afs_uint32 rights)
 {
     register struct cm_aclent *aclp;
 
@@ -190,6 +192,13 @@ long cm_ValidateACLCache(void)
     long count;
     cm_aclent_t * aclp;
 
+    if ( cm_data.aclLRUp == NULL && cm_data.aclLRUEndp != NULL ||
+         cm_data.aclLRUp != NULL && cm_data.aclLRUEndp == NULL) {
+        afsi_log("cm_ValidateACLCache failure: inconsistent LRU pointers");
+        fprintf(stderr, "cm_ValidateACLCache failure: inconsistent LRU pointers\n");
+        return -9;
+    }
+
     for ( aclp = cm_data.aclLRUp, count = 0; aclp;
           aclp = (cm_aclent_t *) osi_QNext(&aclp->q), count++ ) {
         if (aclp->magic != CM_ACLENT_MAGIC) {
