@@ -23,14 +23,6 @@ typedef struct cm_fid {
         unsigned long unique;
 } cm_fid_t;
 
-#if 0
-typedef struct cm_accessCache {
-	osi_queue_t q;			/* queue header */
-        struct cm_user *userp;		/* user having access rights */
-        unsigned long rights;		/* rights */
-} cm_accessCache_t;
-#endif
-
 /* Key used for byte range locking.  Each unique key identifies a
    unique client per cm_scache_t for the purpose of locking. */
 typedef afs_uint64 cm_key_t;
@@ -94,99 +86,121 @@ typedef struct cm_prefetch {		/* last region scanned for prefetching */
 #define CM_SCACHE_MAGIC ('S' | 'C'<<8 | 'A'<<16 | 'C'<<24)
 
 typedef struct cm_scache {
-    osi_queue_t q;              /* lru queue; cm_scacheLock */
-        afs_uint32      magic;
-        struct cm_scache *nextp;	/* next in hash; cm_scacheLock */
-	cm_fid_t fid;
-        afs_uint32 flags;		/* flags; locked by mx */
+    osi_queue_t q;              	/* lru queue; cm_scacheLock */
+    afs_uint32      magic;
+    struct cm_scache *nextp;		/* next in hash; cm_scacheLock */
+    cm_fid_t fid;
+    afs_uint32 flags;			/* flags; locked by mx */
 
-	/* synchronization stuff */
-        osi_mutex_t mx;			/* mutex for this structure */
-        osi_rwlock_t bufCreateLock;	/* read-locked during buffer creation;
-        				 * write-locked to prevent buffers from
-                                         * being created during a truncate op, etc.
-                                         */
-        afs_uint32 refCount;		/* reference count; cm_scacheLock */
-        osi_queueData_t *bufReadsp;	/* queue of buffers being read */
-        osi_queueData_t *bufWritesp;	/* queue of buffers being written */
+    /* synchronization stuff */
+    osi_mutex_t mx;			/* mutex for this structure */
+    osi_rwlock_t bufCreateLock;		/* read-locked during buffer creation;
+                                         * write-locked to prevent buffers from
+					 * being created during a truncate op, etc.
+					 */
+    afs_uint32 refCount;		/* reference count; cm_scacheLock */
+    osi_queueData_t *bufReadsp;		/* queue of buffers being read */
+    osi_queueData_t *bufWritesp;	/* queue of buffers being written */
 
-	/* parent info for ACLs */
-        afs_uint32 parentVnode;		/* parent vnode for ACL callbacks */
-        afs_uint32 parentUnique;	/* for ACL callbacks */
+    /* parent info for ACLs */
+    afs_uint32 parentVnode;		/* parent vnode for ACL callbacks */
+    afs_uint32 parentUnique;		/* for ACL callbacks */
 
-	/* local modification stat */
-        afs_uint32 mask;		/* for clientModTime, length and
-					 * truncPos */
+    /* local modification stat */
+    afs_uint32 mask;			/* for clientModTime, length and
+                                         * truncPos */
 
-	/* file status */
-	afs_uint32 fileType;		/* file type */
-	time_t clientModTime;	        /* mtime */
-        time_t serverModTime;	        /* at server, for concurrent call
-					 * comparisons */
-        osi_hyper_t length;		/* file length */
-	cm_prefetch_t prefetch;		/* prefetch info structure */
-        afs_uint32 unixModeBits;	/* unix protection mode bits */
-        afs_uint32 linkCount;		/* link count */
-        afs_uint32 dataVersion;		/* data version */
-        afs_uint32 owner; 		/* file owner */
-        afs_uint32 group;		/* file owning group */
+    /* file status */
+    afs_uint32 fileType;		/* file type */
+    time_t clientModTime;	        /* mtime */
+    time_t serverModTime;	        /* at server, for concurrent call
+                                         * comparisons */
+    osi_hyper_t length;			/* file length */
+    cm_prefetch_t prefetch;		/* prefetch info structure */
+    afs_uint32 unixModeBits;		/* unix protection mode bits */
+    afs_uint32 linkCount;		/* link count */
+    afs_uint32 dataVersion;		/* data version */
+    afs_uint32 owner; 			/* file owner */
+    afs_uint32 group;			/* file owning group */
+    cm_user_t *creator;			/* user, if new file */
 
-	/* pseudo file status */
-	osi_hyper_t serverLength;	/* length known to server */
+    /* pseudo file status */
+    osi_hyper_t serverLength;		/* length known to server */
 
-	/* aux file status */
-        osi_hyper_t truncPos;		/* file size to truncate to before
-					 * storing data */
+    /* aux file status */
+    osi_hyper_t truncPos;		/* file size to truncate to before
+                                         * storing data */
 
-	/* symlink and mount point info */
-        char mountPointStringp[MOUNTPOINTLEN];	/* the string stored in a mount point;
-        				 * first char is type, then vol name.
+    /* symlink and mount point info */
+    char mountPointStringp[MOUNTPOINTLEN];	/* the string stored in a mount point;
+                                                 * first char is type, then vol name.
                                          * If this is a normal symlink, we store
 					 * the link contents here.
                                          */
-	cm_fid_t  mountRootFid;	        /* mounted on root */
-	time_t    mountRootGen;	        /* time to update mountRootFidp? */
-	cm_fid_t  dotdotFid;		/* parent of volume root */
+    cm_fid_t  mountRootFid;	        /* mounted on root */
+    time_t    mountRootGen;	        /* time to update mountRootFidp? */
+    cm_fid_t  dotdotFid;		/* parent of volume root */
 
-	/* callback info */
-        struct cm_server *cbServerp;	/* server granting callback */
-        time_t cbExpires;		/* time callback expires */
+    /* callback info */
+    struct cm_server *cbServerp;	/* server granting callback */
+    time_t cbExpires;		/* time callback expires */
 
-	/* access cache */
-        long anyAccess;			/* anonymous user's access */
-        struct cm_aclent *randomACLp;	/* access cache entries */
+    /* access cache */
+    long anyAccess;			/* anonymous user's access */
+    struct cm_aclent *randomACLp;	/* access cache entries */
 
     /* file locks */
     afs_int32    serverLock;    /* current lock we have acquired on
                                  * this file.  One of (-1), LockRead
                                  * or LockWrite. [protected by
-                                 * scp->mx]
+                                 * scp->mx].  In the future, this
+                                 * should be replaced by a queue of
+                                 * cm_server_lock_t objects which keep
+                                 * track of lock type, the user for
+                                 * whom the lock was obtained, the
+                                 * dataVersion at the time the lock
+                                 * was asserted last, lastRefreshCycle
+                                 * and lateUpdateTime.
                                  */
     unsigned long lastRefreshCycle; /* protected with cm_scacheLock
                                      * for all scaches. */
+    afs_uint32 lockDataVersion; /* dataVersion of the scp at the time
+                                   the server lock for the scp was
+                                   asserted for this lock the last
+                                   time. */
     osi_queue_t *fileLocksH;    /* queue of locks (head) */
     osi_queue_t *fileLocksT;    /* queue of locks (tail) */
+
     afs_uint32   sharedLocks;   /* number of shared locks on
-                                 * ::fileLocks */
+                                 * ::fileLocks.  This count does not
+                                 * include locks which have
+                                 * CM_FILELOCK_FLAG_CLIENTONLY set. */
+
     afs_uint32   exclusiveLocks; /* number of exclusive locks on
-                                  * ::fileLocks
+                                  * ::fileLocks.  This count does not
+                                  * include locks which have
+                                  * CM_FILELOCK_FLAG_CLIENTONLY set.
                                   */
 	
-	/* volume info */
-        struct cm_volume *volp;		/* volume info; held reference */
-  
-        /* bulk stat progress */
-        osi_hyper_t bulkStatProgress;	/* track bulk stats of large dirs */
+    afs_uint32   clientLocks;   /* number of locks on ::fileLocks that
+                                   have CM_FILELOCK_FLAG_CLIENTONLY
+                                   set. */
+	
+    /* volume info */
+    struct cm_volume *volp;		/* volume info; held reference */
 
-        /* open state */
-        afs_uint16 openReads;		/* open for reading */
-        afs_uint16 openWrites;		/* open for writing */
-        afs_uint16 openShares;		/* open for read excl */
-        afs_uint16 openExcls;		/* open for exclusives */
+    /* bulk stat progress */
+    osi_hyper_t bulkStatProgress;	/* track bulk stats of large dirs */
 
-        /* syncop state */
-        afs_uint32 waitCount;           /* number of threads waiting */
-        afs_uint32 waitRequests;        /* num of thread wait requests */
+    /* open state */
+    afs_uint16 openReads;		/* open for reading */
+    afs_uint16 openWrites;		/* open for writing */
+    afs_uint16 openShares;		/* open for read excl */
+    afs_uint16 openExcls;		/* open for exclusives */
+
+    /* syncop state */
+    afs_uint32 waitCount;           /* number of threads waiting */
+    afs_uint32 waitRequests;        /* num of thread wait requests */
 } cm_scache_t;
 
 /* mask field - tell what has been modified */
@@ -265,9 +279,6 @@ typedef struct cm_scache {
 #define CM_SCACHESYNC_NOWAIT		0x100000/* don't wait for the state,
 						 * just fail */
 
-/* flags for cm_RecycleSCache	*/
-#define CM_SCACHE_RECYCLEFLAG_DESTROY_BUFFERS 	0x1
-
 /* flags for cm_MergeStatus */
 #define CM_MERGEFLAG_FORCE		1	/* check mtime before merging;
 						 * used to see if we're merging
@@ -299,12 +310,12 @@ extern cm_scache_t *cm_GetNewSCache(void);
 extern int cm_FidCmp(cm_fid_t *, cm_fid_t *);
 
 extern long cm_SyncOp(cm_scache_t *, struct cm_buf *, struct cm_user *,
-	struct cm_req *, long, long);
+	struct cm_req *, afs_uint32, afs_uint32);
 
-extern void cm_SyncOpDone(cm_scache_t *, struct cm_buf *, long);
+extern void cm_SyncOpDone(cm_scache_t *, struct cm_buf *, afs_uint32);
 
 extern void cm_MergeStatus(cm_scache_t *, struct AFSFetchStatus *, struct AFSVolSync *,
-	struct cm_user *, int flags);
+	struct cm_user *, afs_uint32 flags);
 
 extern void cm_AFSFidFromFid(struct AFSFid *, cm_fid_t *);
 
@@ -336,6 +347,4 @@ extern long cm_ValidateSCache(void);
 
 extern long cm_ShutdownSCache(void);
 
-extern long cm_RecycleSCache(cm_scache_t *scp, afs_int32 flags);
-
 #endif /*  __CM_SCACHE_H_ENV__ */
