@@ -35,6 +35,16 @@ typedef struct smb {
     unsigned char vdata[1];
 } smb_t;
 
+
+/* reb values */
+#define SMB_FLAGS_SUPPORT_LOCKREAD         0x01
+#define SMB_FLAGS_CLIENT_BUF_AVAIL         0x02
+#define SMB_FLAGS_CASELESS_PATHNAMES       0x08
+#define SMB_FLAGS_CANONICAL_PATHNAMES      0x10
+#define SMB_FLAGS_REQUEST_OPLOCK           0x20
+#define SMB_FLAGS_REQUEST_BATCH_OPLOCK     0x40
+#define SMB_FLAGS_SERVER_TO_CLIENT         0x80           
+
 /* flg2 values */
 
 #define SMB_FLAGS2_KNOWS_LONG_NAMES        0x0001
@@ -43,11 +53,14 @@ typedef struct smb {
 #define SMB_FLAGS2_RESERVED1               0x0008
 #define SMB_FLAGS2_IS_LONG_NAME            0x0040
 #define SMB_FLAGS2_EXT_SEC                 0x0800
-#define SMB_FLAGS2_DFS                     0x1000
+#define SMB_FLAGS2_DFS_PATHNAMES           0x1000
 #define SMB_FLAGS2_PAGING_IO               0x2000
-#define SMB_FLAGS2_ERR_STATUS              0x4000
+#define SMB_FLAGS2_32BIT_STATUS            0x4000
 #define SMB_FLAGS2_UNICODE                 0x8000
 
+#define KNOWS_LONG_NAMES(inp) ((((smb_t *)inp)->flg2 & SMB_FLAGS2_KNOWS_LONG_NAMES)?1:0)
+#define WANTS_DFS_PATHNAMES(inp) ((((smb_t *)inp)->flg2 & SMB_FLAGS2_DFS_PATHNAMES)?1:0)
+
 /* Information Levels */
 #define SMB_INFO_STANDARD               1
 #define SMB_INFO_QUERY_EA_SIZE          2
@@ -78,6 +91,7 @@ typedef struct smb {
 
 /* flags for functions */
 #define SMB_FLAG_CREATE		1	/* create the structure if necessary */
+#define SMB_FLAG_AFSLOGON       2       /* operating on behalf of afslogon.dll */
 
 /* max # of bytes we'll receive in an incoming SMB message */
 /* the maximum is 2^18-1 for NBT and 2^25-1 for Raw transport messages */
@@ -106,6 +120,9 @@ typedef struct smb {
 #define NTNEGOTIATE_CAPABILITY_NTFIND			0x00000200L
 #define NTNEGOTIATE_CAPABILITY_DFS			0x00001000L
 #define NTNEGOTIATE_CAPABILITY_NT_INFO_PASSTHRU		0x00002000L
+#define NTNEGOTIATE_CAPABILITY_LARGE_READX		0x00004000L
+#define NTNEGOTIATE_CAPABILITY_LARGE_WRITEX		0x00008000L
+#define NTNEGOTIATE_CAPABILITY_UNIX     		0x00800000L
 #define NTNEGOTIATE_CAPABILITY_BULK_TRANSFER		0x20000000L
 #define NTNEGOTIATE_CAPABILITY_COMPRESSED		0x40000000L
 #define NTNEGOTIATE_CAPABILITY_EXTENDED_SECURITY	0x80000000L
@@ -150,9 +167,8 @@ typedef struct smb_packet {
 } smb_packet_t;
 
 /* smb_packet flags */
-#define SMB_PACKETFLAG_PROFILE_UPDATE_OK	1
-#define SMB_PACKETFLAG_NOSEND			2
-#define SMB_PACKETFLAG_SUSPENDED		4
+#define SMB_PACKETFLAG_NOSEND			1
+#define SMB_PACKETFLAG_SUSPENDED		2
 
 /* a structure for making Netbios calls; locked by smb_globalLock */
 #define SMB_NCBMAGIC	0x2334344
@@ -176,6 +192,7 @@ typedef struct myncb {
 /* one per virtual circuit */
 typedef struct smb_vc {
     struct smb_vc *nextp;		/* not used */
+    afs_uint32 magic;			/* a magic value to detect bad entries */
     unsigned long refCount;		/* the reference count */
     long flags;			        /* the flags, if any; locked by mx */
     osi_mutex_t mx;			/* the mutex */
@@ -187,8 +204,6 @@ typedef struct smb_vc {
     struct smb_tid *tidsp;		/* the first child in the tid list */
     struct smb_user *usersp;	        /* the first child in the user session list */
     struct smb_fid *fidsp;		/* the first child in the open file list */
-    struct smb_user *justLoggedOut;	/* ready for profile upload? */
-    time_t logoffTime;	                /* tick count when logged off */
     unsigned char errorCount;
     char rname[17];
     int lana;
@@ -196,8 +211,10 @@ typedef struct smb_vc {
     void * secCtx;                      /* security context when negotiating SMB extended auth
                                          * valid when SMB_VCFLAG_AUTH_IN_PROGRESS is set
                                          */
+    unsigned short session;		/* This is the Session Index associated with the NCBs */
 } smb_vc_t;
 
+#define SMB_VC_MAGIC ('S' | 'C'<<8 | 'A'<<16 | 'C'<<24)
 					/* have we negotiated ... */
 #define SMB_VCFLAG_USEV3	1	/* ... version 3 of the protocol */
 #define SMB_VCFLAG_USECORE	2	/* ... the core protocol */
@@ -207,18 +224,22 @@ typedef struct smb_vc {
 #define SMB_VCFLAG_ALREADYDEAD	0x20	/* do not get tokens from this vc */
 #define SMB_VCFLAG_SESSX_RCVD	0x40	/* we received at least one session setups on this vc */
 #define SMB_VCFLAG_AUTH_IN_PROGRESS 0x80 /* a SMB NT extended authentication is in progress */
+#define SMB_VCFLAG_CLEAN_IN_PROGRESS 0x100
 
 /* one per user session */
 typedef struct smb_user {
     struct smb_user *nextp;		/* next sibling */
     unsigned long refCount;		/* ref count */
-    long flags;			        /* flags; locked by mx */
+    afs_uint32 flags;		        /* flags; locked by mx */
     osi_mutex_t mx;
-    long userID;			/* the session identifier */
+    unsigned short userID;		/* the session identifier */
     struct smb_vc *vcp;		        /* back ptr to virtual circuit */
     struct smb_username *unp;           /* user name struct */
+    afs_uint32	delete;			/* ok to del: locked by smb_rctLock */
 } smb_user_t;
 
+#define SMB_USERFLAG_DELETE	    1	/* delete struct when ref count zero */
+
 typedef struct smb_username {
     struct smb_username *nextp;		/* next sibling */
     unsigned long refCount;		/* ref count */
@@ -227,9 +248,25 @@ typedef struct smb_username {
     struct cm_user *userp;		/* CM user structure */
     char *name;			        /* user name */
     char *machine;                      /* machine name */
+    time_t last_logoff_t;		/* most recent logoff time */
 } smb_username_t;
 
-#define SMB_USERFLAG_DELETE	1	/* delete struct when ref count zero */
+/* The SMB_USERNAMEFLAG_AFSLOGON is used to preserve the existence of an 
+ * smb_username_t even when the refCount is zero.  This is used to ensure
+ * that tokens set to a username during the integrated logon process are 
+ * preserved until the SMB Session that will require the tokens is created.
+ * The cm_IoctlSetTokens() function when executed from the Network Provider
+ * connects to the AFS Client Service using the credentials of the machine
+ * and not the user for whom the tokens are being configured. */
+#define SMB_USERNAMEFLAG_AFSLOGON   1
+
+/* The SMB_USERNAMEFLAG_LOGOFF is used to indicate that the user most
+ * recently logged off at 'last_logoff_t'.  The smb_username_t should not
+ * be deleted even if the refCount is zero before 'last_logoff_t' + 
+ * 'smb_LogoffTransferTimeout' if 'smb_LogoffTokenTransfer' is non-zero.
+ * The smb_Daemon() thread is responsible for purging the expired objects */
+
+#define SMB_USERNAMEFLAG_LOGOFF     2
 
 #define SMB_MAX_USERNAME_LENGTH 256
 
@@ -237,17 +274,17 @@ typedef struct smb_username {
 typedef struct smb_tid {
     struct smb_tid *nextp;		/* next sibling */
     unsigned long refCount;
-    long flags;
+    afs_uint32 flags;			/* protected by mx */
     osi_mutex_t mx;			/* for non-tree-related stuff */
     unsigned short tid;		        /* the tid */
     struct smb_vc *vcp;		        /* back ptr */
     struct cm_user *userp;		/* user logged in at the
 					 * tree connect level (base) */
     char *pathname;			/* pathname derived from sharename */
+    afs_uint32	delete;			/* ok to del: locked by smb_rctLock */
 } smb_tid_t;
 
-#define SMB_TIDFLAG_DELETE	1	/* delete struct when ref count zero */
-#define SMB_TIDFLAG_IPC	2 /* IPC$ */
+#define SMB_TIDFLAG_IPC		1 	/* IPC$ */
 
 /* one per process ID */
 typedef struct smb_pid {
@@ -294,11 +331,15 @@ typedef struct smb_ioctl {
 typedef struct smb_fid {
     osi_queue_t q;
     unsigned long refCount;
-    unsigned long flags;
+    afs_uint32 flags;			/* protected by mx */
     osi_mutex_t mx;			/* for non-tree-related stuff */
     unsigned short fid;		        /* the file ID */
     struct smb_vc *vcp;		        /* back ptr */
     struct cm_scache *scp;		/* scache of open file */
+    struct cm_user *userp;              /* user that opened the file
+                                           originally (used to close
+                                           the file if session is
+                                           terminated) */
     long offset;			/* our file pointer */
     smb_ioctl_t *ioctlp;		/* ptr to ioctl structure */
 					/* Under NT, we may need to know the
@@ -313,15 +354,17 @@ typedef struct smb_fid {
     int prev_chunk;			/* previous chunk read */
     int raw_writers;		        /* pending async raw writes */
     EVENT_HANDLE raw_write_event;	/* signal this when raw_writers zero */
+    afs_uint32	delete;			/* ok to del: locked by smb_rctLock */
 } smb_fid_t;
 
 #define SMB_FID_OPENREAD		1	/* open for reading */
 #define SMB_FID_OPENWRITE		2	/* open for writing */
-#define SMB_FID_DELETE			4	/* delete struct on ref count 0 */
+#define SMB_FID_CREATED                 4       /* a new file */
 #define SMB_FID_IOCTL			8	/* a file descriptor for the
 						 * magic ioctl file */
 #define SMB_FID_OPENDELETE		0x10	/* open for deletion (NT) */
 #define SMB_FID_DELONCLOSE		0x20	/* marked for deletion */
+
 /*
  * Now some special flags to work around a bug in NT Client
  */
@@ -330,6 +373,14 @@ typedef struct smb_fid {
 #define SMB_FID_LOOKSLIKECOPY	(SMB_FID_LENGTHSETDONE | SMB_FID_MTIMESETDONE)
 #define SMB_FID_NTOPEN			0x100	/* have dscp and pathp */
 
+#define SMB_FID_SHARE_READ              0x1000
+#define SMB_FID_SHARE_WRITE             0x2000
+
+#define SMB_FID_QLOCK_HIGH              0x0fe00000
+#define SMB_FID_QLOCK_LOW               0x00000000
+#define SMB_FID_QLOCK_LENGTH            1
+#define SMB_FID_QLOCK_PID               0
+
 /*
  * SMB file attributes (32-bit)
  */
@@ -342,13 +393,19 @@ typedef struct smb_fid {
 #define SMB_ATTR_DEVICE         0x0040
 #define SMB_ATTR_NORMAL         0x0080 /* normal file. Only valid if used alone */
 #define SMB_ATTR_TEMPORARY      0x0100
-#define SMB_ATTR_SPARSE_FILE    0x0200
+#define SMB_ATTR_SPARSE_FILE    0x0200 /* used with dfs links */
 #define SMB_ATTR_REPARSE_POINT  0x0400
 #define SMB_ATTR_COMPRESSED     0x0800 /* file or dir is compressed */
 #define SMB_ATTR_OFFLINE        0x1000
 #define SMB_ATTR_NOT_CONTENT_INDEXED 0x2000
 #define SMB_ATTR_ENCRYPTED      0x4000
 
+#define LOCKING_ANDX_SHARED_LOCK        0x01 /* Read-only lock */
+#define LOCKING_ANDX_OPLOCK_RELEASE     0x02 /* Oplock break notification */
+#define LOCKING_ANDX_CHANGE_LOCKTYPE    0x04 /* Change lock type */
+#define LOCKING_ANDX_CANCEL_LOCK        0x08 /* Cancel outstanding request */
+#define LOCKING_ANDX_LARGE_FILES        0x10 /* Large file locking format */
+
 /* for tracking in-progress directory searches */
 typedef struct smb_dirSearch {
     osi_queue_t q;			/* queue of all outstanding cookies */
@@ -356,7 +413,7 @@ typedef struct smb_dirSearch {
     unsigned long refCount;		/* reference count */
     long cookie;			/* value returned to the caller */
     struct cm_scache *scp;		/* vnode of the dir we're searching */
-    time_t lastTime;		        /* last time we used this */
+    time_t lastTime;			/* last time we used this (osi_Time) */
     long flags;			        /* flags (see below);
 					 * locked by smb_globalLock */
     unsigned short attribute;	        /* search attribute
@@ -384,17 +441,33 @@ typedef struct smb_dirListPatch {
  * Note: will not be set if smb_hideDotFiles is false 
  */
 
-/* waiting lock list elements */
+/* individual lock on a waiting lock request */
 typedef struct smb_waitingLock {
-    osi_queue_t q;
-    smb_vc_t *vcp;
-    smb_packet_t *inp;
-    smb_packet_t *outp;
-    time_t timeRemaining;
-    void *lockp;
+    osi_queue_t      q;
+    cm_key_t         key;
+    LARGE_INTEGER    LOffset;
+    LARGE_INTEGER    LLength;
+    cm_file_lock_t * lockp;
+    int              state;
 } smb_waitingLock_t;
 
-extern smb_waitingLock_t *smb_allWaitingLocks;
+#define SMB_WAITINGLOCKSTATE_WAITING 0
+#define SMB_WAITINGLOCKSTATE_DONE    1
+#define SMB_WAITINGLOCKSTATE_ERROR   2
+
+/* waiting lock request */
+typedef struct smb_waitingLockRequest {
+    osi_queue_t   q;
+    smb_vc_t *    vcp;
+    cm_scache_t * scp;
+    smb_packet_t *inp;
+    smb_packet_t *outp;
+    int           lockType;
+    time_t        timeRemaining;
+    smb_waitingLock_t * locks;
+} smb_waitingLockRequest_t;
+
+extern smb_waitingLockRequest_t *smb_allWaitingLocks;
 
 typedef long (smb_proc_t)(smb_vc_t *vcp, smb_packet_t *inp, smb_packet_t *outp);
 
@@ -424,44 +497,57 @@ extern void smb_LargeSearchTimeFromUnixTime(FILETIME *largeTimep, time_t unixTim
 
 extern void smb_UnixTimeFromLargeSearchTime(time_t *unixTimep, FILETIME *largeTimep);
 
-extern void smb_SearchTimeFromUnixTime(time_t *dosTimep, time_t unixTime);
+extern void smb_SearchTimeFromUnixTime(afs_uint32 *searchTimep, time_t unixTime);
 
-extern void smb_UnixTimeFromSearchTime(time_t *unixTimep, time_t searchTime);
+extern void smb_UnixTimeFromSearchTime(time_t *unixTimep, afs_uint32 searchTime);
 
-extern void smb_DosUTimeFromUnixTime(time_t *dosUTimep, time_t unixTime);
+extern void smb_DosUTimeFromUnixTime(afs_uint32 *dosUTimep, time_t unixTime);
 
-extern void smb_UnixTimeFromDosUTime(time_t *unixTimep, time_t dosUTime);
+extern void smb_UnixTimeFromDosUTime(time_t *unixTimep, afs_uint32 dosUTime);
+
+extern void CompensateForSmbClientLastWriteTimeBugs(afs_uint32 *dosTimep);
 
 extern smb_vc_t *smb_FindVC(unsigned short lsn, int flags, int lana);
 
+extern void smb_CleanupDeadVC(smb_vc_t *vcp);
+
 extern void smb_ReleaseVC(smb_vc_t *vcp);
 
 extern void smb_ReleaseVCNoLock(smb_vc_t *vcp);
 
 extern smb_tid_t *smb_FindTID(smb_vc_t *vcp, unsigned short tid, int flags);
 
+extern void smb_HoldTIDNoLock(smb_tid_t *tidp);
+
 extern void smb_ReleaseTID(smb_tid_t *tidp);
 
 extern smb_user_t *smb_FindUID(smb_vc_t *vcp, unsigned short uid, int flags);
 
-extern smb_username_t *smb_FindUserByName(char *usern, char *machine, int flags);
+extern smb_username_t *smb_FindUserByName(char *usern, char *machine, afs_uint32 flags);
 
 extern smb_user_t *smb_FindUserByNameThisSession(smb_vc_t *vcp, char *usern); 
 
-extern smb_username_t *smb_FindUserByName(char *usern, char *machine, int flags);
+extern void smb_ReleaseUsername(smb_username_t *unp);
 
-extern smb_user_t *smb_FindUserByNameThisSession(smb_vc_t *vcp, char *usern);
+extern void smb_HoldUIDNoLock(smb_user_t *uidp);
 
 extern void smb_ReleaseUID(smb_user_t *uidp);
 
-extern cm_user_t *smb_GetUser(smb_vc_t *vcp, smb_packet_t *inp);
+extern cm_user_t *smb_GetUserFromVCP(smb_vc_t *vcp, smb_packet_t *inp);
+
+extern cm_user_t *smb_GetUserFromUID(smb_user_t *uidp);
 
 extern long smb_LookupTIDPath(smb_vc_t *vcp, unsigned short tid, char ** tidPathp);
 
 extern smb_fid_t *smb_FindFID(smb_vc_t *vcp, unsigned short fid, int flags);
 
+extern void smb_HoldFIDNoLock(smb_fid_t *fidp);
+
 extern void smb_ReleaseFID(smb_fid_t *fidp);
 
+extern long smb_CloseFID(smb_vc_t *vcp, smb_fid_t *fidp, cm_user_t *userp,
+                         afs_uint32 dosTime);
+
 extern int smb_FindShare(smb_vc_t *vcp, smb_user_t *uidp, char *shareName, char **pathNamep);
 
 extern int smb_FindShareCSCPolicy(char *shareName);
@@ -519,10 +605,6 @@ extern void smb_HoldVCNoLock(smb_vc_t *vcp);
 
 /* some globals, too */
 extern char *smb_localNamep;
-extern int loggedOut;
-extern time_t loggedOutTime;
-extern char *loggedOutName;
-extern smb_user_t *loggedOutUserp;
 
 extern osi_log_t *smb_logp;
 
@@ -626,4 +708,18 @@ cm_user_t *smb_FindOrCreateUser(smb_vc_t *vcp, char *usern);
 #ifdef NOTSERVICE
 extern void smb_LogPacket(smb_packet_t *packet);
 #endif /* NOTSERVICE */
+
+#ifndef MSV1_0_OPTION_ALLOW_BLANK_PASSWORD
+#define MSV1_0_OPTION_ALLOW_BLANK_PASSWORD      0x1
+#define MSV1_0_OPTION_DISABLE_ADMIN_LOCKOUT     0x2
+#define MSV1_0_OPTION_DISABLE_FORCE_GUEST       0x4
+#define MSV1_0_OPTION_TRY_CACHE_FIRST           0x10
+
+typedef struct _MSV1_0_SETPROCESSOPTION_REQUEST {
+    MSV1_0_PROTOCOL_MESSAGE_TYPE MessageType;
+    ULONG ProcessOptions;
+    BOOLEAN DisableOptions;
+} MSV1_0_SETPROCESSOPTION_REQUEST, *PMSV1_0_SETPROCESSOPTION_REQUEST; 
+#endif
+
 #endif /* whole file */
