@@ -81,13 +81,13 @@ protected:
         STDMETHOD(QueryContextMenu)( HMENU hmenu,UINT indexMenu,UINT idCmdFirst,
             UINT idCmdLast,UINT uFlags);
         STDMETHOD(InvokeCommand)(LPCMINVOKECOMMANDINFO lpici);
-        STDMETHOD(GetCommandString)(UINT idCmd,UINT uType,UINT* pwReserved,LPSTR pszName,
+        STDMETHOD(GetCommandString)(UINT_PTR idCmd,UINT uType,UINT* pwReserved,LPSTR pszName,
             UINT cchMax);
     END_INTERFACE_PART(MenuExt)
 
     // IShellExtInit interface
     BEGIN_INTERFACE_PART(ShellInit, IShellExtInit)
-        STDMETHOD(Initialize)(LPCITEMIDLIST pidlFolder,LPDATAOBJECT lpdobj, HKEY hkeyProgID);
+        STDMETHOD(Initialize)(LPCITEMIDLIST pidlFolder,IDataObject *lpdobj, HKEY hkeyProgID);
     END_INTERFACE_PART(ShellInit)
 
     BEGIN_INTERFACE_PART(IconExt, IShellIconOverlayIdentifier)
