@@ -15,7 +15,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/LINUX/osi_module.c,v 1.72 2006/01/14 08:00:01 rra Exp $");
+    ("$Header: /cvs/openafs/src/afs/LINUX/osi_module.c,v 1.74 2006/03/09 06:06:34 shadow Exp $");
 
 #include <linux/module.h> /* early to avoid printf->printk mapping */
 #include "afs/sysincludes.h"
@@ -48,7 +48,9 @@ extern struct file_system_type afs_fs_type;
 static long get_page_offset(void);
 #endif
 
-#if defined(AFS_LINUX24_ENV)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
+DEFINE_MUTEX(afs_global_lock);
+#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
 DECLARE_MUTEX(afs_global_lock);
 #else
 struct semaphore afs_global_lock = MUTEX;
@@ -634,6 +636,8 @@ afs_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
 #ifdef AFS_LINUX26_ENV 
 #ifdef AFS_S390X_LINUX26_ENV
     if (test_thread_flag(TIF_31BIT))
+#elif AFS_AMD64_LINUX20_ENV
+    if (test_thread_flag(TIF_IA32))
 #else
     if (test_thread_flag(TIF_32BIT))
 #endif /* AFS_S390X_LINUX26_ENV */
