@@ -11,7 +11,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_call.c,v 1.74.2.6 2005/03/20 14:54:11 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_call.c,v 1.86 2006/01/26 16:00:34 shadow Exp $");
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
 #include "afsincludes.h"	/* Afs-based standard headers */
@@ -31,7 +31,7 @@ RCSID
 #endif
 
 
-#if	defined(AFS_AIX_ENV) || defined(AFS_SGI_ENV) || defined(AFS_SUN_ENV) || defined(AFS_HPUX_ENV)
+#if defined(AFS_SUN5_ENV) || defined(AFS_AIX_ENV) || defined(AFS_SGI_ENV) || defined(AFS_HPUX_ENV)
 #define	AFS_MINBUFFERS	100
 #else
 #define	AFS_MINBUFFERS	50
@@ -50,7 +50,6 @@ afs_uint32 rx_bindhost;
 
 #if defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
 kmutex_t afs_global_lock;
-kmutex_t afs_rxglobal_lock;
 #endif
 
 #if defined(AFS_SGI_ENV) && !defined(AFS_SGI64_ENV)
@@ -61,9 +60,13 @@ long afs_global_owner;
 simple_lock_data_t afs_global_lock;
 #endif
 
-#if defined(AFS_DARWIN_ENV)
+#if defined(AFS_DARWIN_ENV) 
+#ifdef AFS_DARWIN80_ENV
+lck_mtx_t  *afs_global_lock;
+#else
 struct lock__bsd__ afs_global_lock;
 #endif
+#endif
 
 #if defined(AFS_XBSD_ENV) && !defined(AFS_FBSD50_ENV)
 struct lock afs_global_lock;
@@ -146,6 +149,133 @@ afs_InitSetup(int preallocs)
 
     return code;
 }
+#if defined(AFS_DARWIN80_ENV)
+struct afsd_thread_info {
+    unsigned long parm;
+};
+static int
+afsd_thread(int *rock)
+{
+    struct afsd_thread_info *arg = (struct afsd_thread_info *)rock;
+    unsigned long parm = arg->parm;
+
+    switch (parm) {
+    case AFSOP_START_RXCALLBACK:
+	AFS_GLOCK();
+	wakeup(arg);
+	afs_CB_Running = 1;
+	while (afs_RX_Running != 2)
+	    afs_osi_Sleep(&afs_RX_Running);
+	afs_RXCallBackServer();
+	AFS_GUNLOCK();
+	thread_terminate(current_thread());
+	break;
+    case AFSOP_START_AFS:
+	AFS_GLOCK();
+	wakeup(arg);
+	AFS_Running = 1;
+	while (afs_initState < AFSOP_START_AFS)
+	    afs_osi_Sleep(&afs_initState);
+	afs_initState = AFSOP_START_BKG;
+	afs_osi_Wakeup(&afs_initState);
+	afs_Daemon();
+	AFS_GUNLOCK();
+	thread_terminate(current_thread());
+	break;
+    case AFSOP_START_BKG:
+	AFS_GLOCK();
+	wakeup(arg);
+	while (afs_initState < AFSOP_START_BKG)
+	    afs_osi_Sleep(&afs_initState);
+	if (afs_initState < AFSOP_GO) {
+	    afs_initState = AFSOP_GO;
+	    afs_osi_Wakeup(&afs_initState);
+	}
+	afs_BackgroundDaemon();
+	AFS_GUNLOCK();
+	thread_terminate(current_thread());
+	break;
+    case AFSOP_START_TRUNCDAEMON:
+	AFS_GLOCK();
+	wakeup(arg);
+	while (afs_initState < AFSOP_GO)
+	    afs_osi_Sleep(&afs_initState);
+	afs_CacheTruncateDaemon();
+	AFS_GUNLOCK();
+	thread_terminate(current_thread());
+	break;
+    case AFSOP_START_CS:
+	AFS_GLOCK();
+	wakeup(arg);
+	afs_CheckServerDaemon();
+	AFS_GUNLOCK();
+	thread_terminate(current_thread());
+	break;
+    case AFSOP_RXEVENT_DAEMON:
+	AFS_GLOCK();
+	wakeup(arg);
+	while (afs_initState < AFSOP_START_BKG)
+	    afs_osi_Sleep(&afs_initState);
+	afs_rxevent_daemon();
+	AFS_GUNLOCK();
+	thread_terminate(current_thread());
+	break;
+    case AFSOP_RXLISTENER_DAEMON:
+	AFS_GLOCK();
+	wakeup(arg);
+	afs_initState = AFSOP_START_AFS;
+	afs_osi_Wakeup(&afs_initState);
+	afs_RX_Running = 2;
+	afs_osi_Wakeup(&afs_RX_Running);
+	afs_osi_RxkRegister();
+	rxk_Listener();
+	AFS_GUNLOCK();
+	thread_terminate(current_thread());
+	break;
+    default:
+	printf("Unknown op %ld in StartDaemon()\n", (long)parm);
+	break;
+    }
+}
+
+void
+afs_DaemonOp(long parm, long parm2, long parm3, long parm4, long parm5,
+	     long parm6)
+{
+    int code;
+    struct afsd_thread_info info;
+    thread_t thread;
+
+    if (parm == AFSOP_START_RXCALLBACK) {
+	if (afs_CB_Running)
+	    return;
+    } else if (parm == AFSOP_RXLISTENER_DAEMON) {
+	if (afs_RX_Running)
+	    return;
+	afs_RX_Running = 1;
+	code = afs_InitSetup(parm2);
+	if (parm3) {
+	    rx_enablePeerRPCStats();
+	}
+	if (parm4) {
+	    rx_enableProcessRPCStats();
+	}
+	if (code)
+	    return;
+    } else if (parm == AFSOP_START_AFS) {
+	if (AFS_Running)
+	    return;
+    }				/* other functions don't need setup in the parent */
+    info.parm = parm;
+    kernel_thread_start((thread_continue_t)afsd_thread, &info, &thread);
+    AFS_GUNLOCK();
+    /* we need to wait cause we passed stack pointers around.... */
+    msleep(&info, NULL, PVFS, "afs_DaemonOp", NULL);
+    AFS_GLOCK();
+    thread_deallocate(thread);
+}
+#endif
+
 
 #if defined(AFS_LINUX24_ENV) && defined(COMPLETION_H_EXISTS)
 struct afsd_thread_info {
@@ -374,7 +504,10 @@ afs_syscall_call(parm, parm2, parm3, parm4, parm5, parm6)
 #endif
     }
     AFS_GLOCK();
-#if defined(AFS_LINUX24_ENV) && defined(COMPLETION_H_EXISTS) && !defined(UKERNEL)
+#ifdef AFS_DARWIN80_ENV
+    put_vfs_context();
+#endif
+#if ((defined(AFS_LINUX24_ENV) && defined(COMPLETION_H_EXISTS)) || defined(AFS_DARWIN80_ENV)) && !defined(UKERNEL)
     if (parm < AFSOP_ADDCELL || parm == AFSOP_RXEVENT_DAEMON
 	|| parm == AFSOP_RXLISTENER_DAEMON) {
 	afs_DaemonOp(parm, parm2, parm3, parm4, parm5, parm6);
@@ -505,7 +638,7 @@ afs_syscall_call(parm, parm2, parm3, parm4, parm5, parm6)
 	while (!afs_InitSetup_done)
 	    afs_osi_Sleep(&afs_InitSetup_done);
 
-#if defined(AFS_SUN_ENV) || defined(AFS_SGI_ENV) || defined(AFS_HPUX_ENV) || defined(AFS_LINUX20_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#if defined(AFS_SGI_ENV) || defined(AFS_HPUX_ENV) || defined(AFS_LINUX20_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV) || defined(AFS_SUN5_ENV)
 	temp = AFS_MINBUFFERS;	/* Should fix this soon */
 #else
 	/* number of 2k buffers we could get from all of the buffer space */
@@ -517,6 +650,18 @@ afs_syscall_call(parm, parm2, parm3, parm4, parm5, parm6)
 	DInit(temp);
 	afs_rootFid.Fid.Volume = 0;
 	code = 0;
+    } else if (parm == AFSOP_BUCKETPCT) {
+	/* need to enable this now, will disable again before GO
+	   if we don't have 100% */
+	splitdcache = 1;
+	switch (parm2) {
+	case 1:
+	    afs_tpct1 = parm3;
+	    break;
+	case 2:
+	    afs_tpct2 = parm3;
+	    break;
+	}           
     } else if (parm == AFSOP_ADDCELL) {
 	/* add a cell.  Parameter 2 is 8 hosts (in net order),  parm 3 is the null-terminated
 	 * name.  Parameter 4 is the length of the name, including the null.  Parm 5 is the
@@ -642,11 +787,17 @@ afs_syscall_call(parm, parm2, parm3, parm4, parm5, parm6)
 	while (afs_initState < AFSOP_START_BKG)
 	    afs_osi_Sleep(&afs_initState);
 
+#ifdef AFS_DARWIN80_ENV
+    get_vfs_context();
+#endif
 	/* do it by inode */
 #ifdef AFS_SGI62_ENV
 	ainode = (ainode << 32) | (parm3 & 0xffffffff);
 #endif
 	code = afs_InitCacheFile(NULL, ainode);
+#ifdef AFS_DARWIN80_ENV
+    put_vfs_context();
+#endif
     } else if (parm == AFSOP_ROOTVOLUME) {
 	/* wait for basic init */
 	while (afs_initState < AFSOP_START_BKG)
@@ -673,6 +824,9 @@ afs_syscall_call(parm, parm2, parm3, parm4, parm5, parm6)
 	if (!code) {
 	    tbuffer[AFS_SMALLOCSIZ - 1] = '\0';	/* null-terminate the name */
 	    /* We have the cache dir copied in.  Call the cache init routine */
+#ifdef AFS_DARWIN80_ENV
+    get_vfs_context();
+#endif
 	    if (parm == AFSOP_CACHEFILE)
 		code = afs_InitCacheFile(tbuffer, 0);
 	    else if (parm == AFSOP_CACHEINFO)
@@ -681,6 +835,9 @@ afs_syscall_call(parm, parm2, parm3, parm4, parm5, parm6)
 		code = afs_InitVolumeInfo(tbuffer);
 	    else if (parm == AFSOP_CELLINFO)
 		code = afs_InitCellInfo(tbuffer);
+#ifdef AFS_DARWIN80_ENV
+    put_vfs_context();
+#endif
 	}
 	osi_FreeSmallSpace(tbuffer);
     } else if (parm == AFSOP_GO) {
@@ -693,8 +850,15 @@ afs_syscall_call(parm, parm2, parm3, parm4, parm5, parm6)
 	    afs_osi_Sleep(&afs_initState);
 	afs_initState = 101;
 	afs_setTime = parm2;
+	if (afs_tpct1 + afs_tpct2 != 100) {
+	    afs_tpct1 = 0;
+	    afs_tpct2 = 0;
+	    splitdcache = 0;
+	} else {        
+	    splitdcache = 1;
+	}
 	afs_osi_Wakeup(&afs_initState);
-#if	(!defined(AFS_NONFSTRANS) && !defined(AFS_DEC_ENV)) || defined(AFS_AIX_IAUTH_ENV)
+#if	(!defined(AFS_NONFSTRANS)) || defined(AFS_AIX_IAUTH_ENV)
 	afs_nfsclient_init();
 #endif
 	printf("found %d non-empty cache files (%d%%).\n",
@@ -821,10 +985,10 @@ afs_syscall_call(parm, parm2, parm3, parm4, parm5, parm6)
 	i = rxi_Findcbi(parm2);
 	mtu = ((i == -1) ? htonl(1500) : afs_cb_interface.mtu[i]);
 #else /* AFS_USERSPACE_IP_ADDR */
-	struct ifnet *tifnp;
+	AFS_IFNET_T tifnp;
 
 	tifnp = rxi_FindIfnet(parm2, NULL);	/*  make iterative */
-	mtu = (tifnp ? tifnp->if_mtu : htonl(1500));
+	mtu = (tifnp ? ifnet_mtu(tifnp) : htonl(1500));
 #endif /* else AFS_USERSPACE_IP_ADDR */
 #endif /* !AFS_SUN5_ENV */
 	if (!code)
@@ -858,7 +1022,7 @@ afs_syscall_call(parm, parm2, parm3, parm4, parm5, parm6)
 	    code = -1;
 	}
 #else /* AFS_USERSPACE_IP_ADDR */
-	struct ifnet *tifnp;
+	AFS_IFNET_T tifnp;
 
 	tifnp = rxi_FindIfnet(parm2, &mask);	/* make iterative */
 	if (!tifnp)
@@ -911,6 +1075,9 @@ afs_syscall_call(parm, parm2, parm3, parm4, parm5, parm6)
 	code = EINVAL;
 
   out:
+#ifdef AFS_DARWIN80_ENV /* to balance the put in afs3_syscall() */
+    get_vfs_context();
+#endif
     AFS_GUNLOCK();
 #ifdef AFS_LINUX20_ENV
     return -code;
@@ -1148,7 +1315,9 @@ copyin_iparam(caddr_t cmarg, struct iparam *dst)
 #if defined(AFS_LINUX_64BIT_KERNEL) && !defined(AFS_ALPHA_LINUX20_ENV) && !defined(AFS_IA64_LINUX20_ENV)
     struct iparam32 dst32;
 
-#ifdef AFS_SPARC64_LINUX24_ENV
+#ifdef AFS_SPARC64_LINUX26_ENV
+    if (test_thread_flag(TIF_32BIT))
+#elif AFS_SPARC64_LINUX24_ENV
     if (current->thread.flags & SPARC_FLAG_32BIT)
 #elif defined(AFS_SPARC64_LINUX20_ENV)
     if (current->tss.flags & SPARC_FLAG_32BIT)
@@ -1227,7 +1396,7 @@ afs3_syscall(p, args, retval)
      struct proc *p;
 #endif
      void *args;
-     int *retval;
+     long *retval;
 {
     register struct a {
 	long syscall;
@@ -1276,11 +1445,7 @@ Afs_syscall()
     } *uap = (struct a *)u.u_ap;
 #else /* UKERNEL */
 int
-#if defined(AFS_SUN_ENV) && !defined(AFS_SUN5_ENV)
-afs_syscall()
-#else
 Afs_syscall()
-#endif				/* SUN && !SUN5 */
 {
     register struct a {
 	long syscall;
@@ -1292,9 +1457,7 @@ Afs_syscall()
 	long parm6;
     } *uap = (struct a *)u.u_ap;
 #endif /* UKERNEL */
-#if  defined(AFS_DEC_ENV)
-    int *retval = &u.u_r.r_val1;
-#elif defined(AFS_HPUX_ENV)
+#if defined(AFS_HPUX_ENV)
     long *retval = &u.u_rval1;
 #else
     int *retval = &u.u_rval1;
@@ -1332,7 +1495,12 @@ Afs_syscall()
 })
 
 
-	if (current->thread.flags & SPARC_FLAG_32BIT) {
+#ifdef AFS_SPARC64_LINUX26_ENV
+	if (test_thread_flag(TIF_32BIT))
+#else
+	if (current->thread.flags & SPARC_FLAG_32BIT)
+#endif
+	{
 	    AFS_COPYIN((char *)parm4, (char *)eparm32, sizeof(eparm32), code);
 	    eparm[0] = AA(eparm32[0]);
 	    eparm[1] = AA(eparm32[1]);
@@ -1350,7 +1518,10 @@ Afs_syscall()
 	uap->parm6 = 0;
     }
 #endif
-
+#if defined(AFS_DARWIN80_ENV)
+    get_vfs_context();
+    osi_Assert(*retval == 0);
+#endif
 #if defined(AFS_HPUX_ENV)
     /*
      * There used to be code here (duplicated from osi_Init()) for
@@ -1397,6 +1568,10 @@ Afs_syscall()
 	code =
 	    afs_syscall_pioctl(uap->parm1, uap->parm2, uap->parm3, uap->parm4,
 			       p->td_ucred);
+#elif defined(AFS_DARWIN80_ENV)
+	code =
+	    afs_syscall_pioctl(uap->parm1, uap->parm2, uap->parm3, uap->parm4,
+			       kauth_cred_get());
 #elif defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 	code =
 	    afs_syscall_pioctl(uap->parm1, uap->parm2, uap->parm3, uap->parm4,
@@ -1485,6 +1660,9 @@ Afs_syscall()
 #endif
     }
 
+#if defined(AFS_DARWIN80_ENV)
+    put_vfs_context();
+#endif
 #ifdef AFS_LINUX20_ENV
     code = -code;
     unlock_kernel();
@@ -1632,11 +1810,8 @@ afs_shutdown(void)
     shutdown_vfsops();
     shutdown_exporter();
     shutdown_memcache();
-#if !defined(AFS_NONFSTRANS) || defined(AFS_AIX_IAUTH_ENV)
-#if !defined(AFS_DEC_ENV) && !defined(AFS_OSF_ENV)
-    /* this routine does not exist in Ultrix systems... 93.01.19 */
+#if (!defined(AFS_NONFSTRANS) || defined(AFS_AIX_IAUTH_ENV)) && !defined(AFS_OSF_ENV)
     shutdown_nfsclnt();
-#endif /* AFS_DEC_ENV */
 #endif
     shutdown_afstest();
     /* The following hold the cm stats */
@@ -1671,7 +1846,7 @@ afs_shutdown_BKG(void)
 }
 
 
-#if defined(AFS_ALPHA_ENV) || defined(AFS_SGI61_ENV)
+#if defined(AFS_OSF_ENV) || defined(AFS_SGI61_ENV)
 /* For SGI 6.2, this can is changed to 1 if it's a 32 bit kernel. */
 #if defined(AFS_SGI62_ENV) && defined(KERNEL) && !defined(_K64U64)
 int afs_icl_sizeofLong = 1;
@@ -2106,16 +2281,16 @@ afs_icl_AppendString(struct afs_icl_log *logp, char *astr)
         (lp)->logElements++; \
     MACRO_END
 
-#if defined(AFS_ALPHA_ENV) || (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG==64)) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
+#if defined(AFS_OSF_ENV) || (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG==64)) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
 #define ICL_APPENDLONG(lp, x) \
     MACRO_BEGIN \
 	ICL_APPENDINT32((lp), ((x) >> 32) & 0xffffffffL); \
 	ICL_APPENDINT32((lp), (x) & 0xffffffffL); \
     MACRO_END
 
-#else /* AFS_ALPHA_ENV */
+#else /* AFS_OSF_ENV */
 #define ICL_APPENDLONG(lp, x) ICL_APPENDINT32((lp), (x))
-#endif /* AFS_ALPHA_ENV */
+#endif /* AFS_OSF_ENV */
 
 /* routine to tell whether we're dealing with the address or the
  * object itself
@@ -2260,10 +2435,10 @@ afs_icl_AppendRecord(register struct afs_icl_log *logp, afs_int32 op,
 	    ICL_APPENDINT32(logp, (afs_int32) ((afs_int32 *) p1)[2]);
 	    ICL_APPENDINT32(logp, (afs_int32) ((afs_int32 *) p1)[3]);
 	}
-#if defined(AFS_ALPHA_ENV) || (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG==64)) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
+#if defined(AFS_OSF_ENV) || (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG==64)) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
 	else if (t1 == ICL_TYPE_INT32)
 	    ICL_APPENDINT32(logp, (afs_int32) p1);
-#endif /* AFS_ALPHA_ENV */
+#endif /* AFS_OSF_ENV */
 	else
 	    ICL_APPENDLONG(logp, p1);
     }
@@ -2300,10 +2475,10 @@ afs_icl_AppendRecord(register struct afs_icl_log *logp, afs_int32 op,
 	    ICL_APPENDINT32(logp, (afs_int32) ((afs_int32 *) p2)[2]);
 	    ICL_APPENDINT32(logp, (afs_int32) ((afs_int32 *) p2)[3]);
 	}
-#if defined(AFS_ALPHA_ENV) || (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG==64)) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
+#if defined(AFS_OSF_ENV) || (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG==64)) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
 	else if (t2 == ICL_TYPE_INT32)
 	    ICL_APPENDINT32(logp, (afs_int32) p2);
-#endif /* AFS_ALPHA_ENV */
+#endif /* AFS_OSF_ENV */
 	else
 	    ICL_APPENDLONG(logp, p2);
     }
@@ -2340,10 +2515,10 @@ afs_icl_AppendRecord(register struct afs_icl_log *logp, afs_int32 op,
 	    ICL_APPENDINT32(logp, (afs_int32) ((afs_int32 *) p3)[2]);
 	    ICL_APPENDINT32(logp, (afs_int32) ((afs_int32 *) p3)[3]);
 	}
-#if defined(AFS_ALPHA_ENV) || (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG==64)) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
+#if defined(AFS_OSF_ENV) || (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG==64)) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
 	else if (t3 == ICL_TYPE_INT32)
 	    ICL_APPENDINT32(logp, (afs_int32) p3);
-#endif /* AFS_ALPHA_ENV */
+#endif /* AFS_OSF_ENV */
 	else
 	    ICL_APPENDLONG(logp, p3);
     }
@@ -2380,10 +2555,10 @@ afs_icl_AppendRecord(register struct afs_icl_log *logp, afs_int32 op,
 	    ICL_APPENDINT32(logp, (afs_int32) ((afs_int32 *) p4)[2]);
 	    ICL_APPENDINT32(logp, (afs_int32) ((afs_int32 *) p4)[3]);
 	}
-#if defined(AFS_ALPHA_ENV) || (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG==64)) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
+#if defined(AFS_OSF_ENV) || (defined(AFS_SGI61_ENV) && (_MIPS_SZLONG==64)) || (defined(AFS_AIX51_ENV) && defined(AFS_64BIT_KERNEL))
 	else if (t4 == ICL_TYPE_INT32)
 	    ICL_APPENDINT32(logp, (afs_int32) p4);
-#endif /* AFS_ALPHA_ENV */
+#endif /* AFS_OSF_ENV */
 	else
 	    ICL_APPENDLONG(logp, p4);
     }
