@@ -11,7 +11,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_daemons.c,v 1.41 2006/02/13 18:29:26 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_daemons.c,v 1.43 2006/03/30 17:13:39 rees Exp $");
 
 #ifdef AFS_AIX51_ENV
 #define __FULL_PROTO
@@ -46,14 +46,34 @@ afs_int32 afs_gcpags = AFS_GCPAGS;
 afs_int32 afs_gcpags_procsize = 0;
 
 afs_int32 afs_CheckServerDaemonStarted = 0;
-#ifdef DEFAULT_PROBE_INTERVAL
-afs_int32 PROBE_INTERVAL = DEFAULT_PROBE_INTERVAL;	/* overridding during compile */
-#else
-afs_int32 PROBE_INTERVAL = 180;	/* default to 3 min */
+#ifndef DEFAULT_PROBE_INTERVAL
+#define DEFAULT_PROBE_INTERVAL 180	/* default to 3 min */
 #endif
+afs_int32 afs_probe_interval = DEFAULT_PROBE_INTERVAL;
+afs_int32 afs_probe_all_interval = 600;
+afs_int32 afs_nat_probe_interval = 60;
 
-#define PROBE_WAIT() (1000 * (PROBE_INTERVAL - ((afs_random() & 0x7fffffff) \
-		      % (PROBE_INTERVAL/2))))
+#define PROBE_WAIT() (1000 * (afs_probe_interval - ((afs_random() & 0x7fffffff) \
+		      % (afs_probe_interval/2))))
+
+void
+afs_SetCheckServerNATmode(int isnat)
+{
+    static afs_int32 old_intvl, old_all_intvl;
+    static int wasnat;
+
+    if (isnat && !wasnat) {
+	old_intvl = afs_probe_interval;
+	old_all_intvl = afs_probe_all_interval;
+	afs_probe_interval = afs_nat_probe_interval;
+	afs_probe_all_interval = afs_nat_probe_interval;
+	afs_osi_CancelWait(&AFS_CSWaitHandler);
+    } else if (!isnat && wasnat) {
+	afs_probe_interval = old_intvl;
+	afs_probe_all_interval = old_all_intvl;
+    }
+    wasnat = isnat;
+}
 
 void
 afs_CheckServerDaemon(void)
@@ -75,13 +95,13 @@ afs_CheckServerDaemon(void)
 	}
 
 	now = osi_Time();
-	if (PROBE_INTERVAL + lastCheck <= now) {
+	if (afs_probe_interval + lastCheck <= now) {
 	    afs_CheckServers(1, NULL);	/* check down servers */
 	    lastCheck = now = osi_Time();
 	}
 
-	if (600 + last10MinCheck <= now) {
-	    afs_Trace1(afs_iclSetp, CM_TRACE_PROBEUP, ICL_TYPE_INT32, 600);
+	if (afs_probe_all_interval + last10MinCheck <= now) {
+	    afs_Trace1(afs_iclSetp, CM_TRACE_PROBEUP, ICL_TYPE_INT32, afs_probe_all_interval);
 	    afs_CheckServers(0, NULL);
 	    last10MinCheck = now = osi_Time();
 	}
@@ -93,9 +113,9 @@ afs_CheckServerDaemon(void)
 	}
 
 	/* Compute time to next probe. */
-	delay = PROBE_INTERVAL + lastCheck;
-	if (delay > 600 + last10MinCheck)
-	    delay = 600 + last10MinCheck;
+	delay = afs_probe_interval + lastCheck;
+	if (delay > afs_probe_all_interval + last10MinCheck)
+	    delay = afs_probe_all_interval + last10MinCheck;
 	delay -= now;
 	if (delay < 1)
 	    delay = 1;
@@ -197,7 +217,7 @@ afs_Daemon(void)
 		cs_warned = 1;
 		printf("Please install afsd with check server daemon.\n");
 	    }
-	    if (lastNMinCheck + PROBE_INTERVAL < now) {
+	    if (lastNMinCheck + afs_probe_interval < now) {
 		/* only check down servers */
 		afs_CheckServers(1, NULL);
 		lastNMinCheck = now;
