@@ -33,7 +33,7 @@
 #include "afs/param.h"
 
 RCSID
-    ("$Header: /cvs/openafs/src/afs/afs_server.c,v 1.33.2.12 2007/12/11 21:31:52 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afs/afs_server.c,v 1.43 2005/11/14 06:00:38 jaltman Exp $");
 
 #include "afs/stds.h"
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
@@ -488,42 +488,6 @@ afs_CountServers(void)
 }				/*afs_CountServers */
 
 
-void
-ForceAllNewConnections()
-{
-    int srvAddrCount;
-    struct srvAddr **addrs;
-    struct srvAddr *sa;
-    afs_int32 i, j;
-
-    ObtainReadLock(&afs_xserver);	/* Necessary? */
-    ObtainReadLock(&afs_xsrvAddr);
-
-    srvAddrCount = 0;
-    for (i = 0; i < NSERVERS; i++) {
-	for (sa = afs_srvAddrs[i]; sa; sa = sa->next_bkt) {
-	    srvAddrCount++;
-	}
-    }
-
-    addrs = afs_osi_Alloc(srvAddrCount * sizeof(*addrs));
-    j = 0;
-    for (i = 0; i < NSERVERS; i++) {
-	for (sa = afs_srvAddrs[i]; sa; sa = sa->next_bkt) {
-	    if (j >= srvAddrCount)
-		break;
-	    addrs[j++] = sa;
-	}
-    }
-
-    ReleaseReadLock(&afs_xsrvAddr);
-    ReleaseReadLock(&afs_xserver);
-    for (i = 0; i < j; i++) {
-        sa = addrs[i];
-	ForceNewConnections(sa);
-    }
-}
-
 /* check down servers (if adown), or running servers (if !adown) */
 void
 afs_CheckServers(int adown, struct cell *acellp)
@@ -1129,7 +1093,6 @@ afsi_SetServerIPRank(struct srvAddr *sa, afs_int32 addr,
 	    sa->sa_iprank = afs_min(sa->sa_iprank, MED);
 	}
     }
-    return;
 }
 #else /* AFS_USERSPACE_IP_ADDR */
 #if (! defined(AFS_SUN5_ENV)) && !defined(AFS_DARWIN60_ENV) && defined(USEIFADDR)
@@ -1274,74 +1237,23 @@ static int afs_SetServerPrefs(struct srvAddr *sa) {
 #else				/* AFS_USERSPACE_IP_ADDR */
 #if	defined(AFS_SUN5_ENV)
 #ifdef AFS_SUN510_ENV
-    int i = 0;
+    ill_walk_context_t ctx;
 #else
     extern struct ill_s *ill_g_headp;
     long *addr = (long *)ill_g_headp;
+#endif
     ill_t *ill;
     ipif_t *ipif;
-#endif
     int subnet, subnetmask, net, netmask;
 
     if (sa)
 	  sa->sa_iprank = 0;
 #ifdef AFS_SUN510_ENV
-    rw_enter(&afsifinfo_lock, RW_READER);
-
-    for (i = 0; (afsifinfo[i].ipaddr != NULL) && (i < ADDRSPERSITE); i++) {
-
-	if (IN_CLASSA(afsifinfo[i].ipaddr)) {
-	    netmask = IN_CLASSA_NET;
-	} else if (IN_CLASSB(afsifinfo[i].ipaddr)) {
-	    netmask = IN_CLASSB_NET;
-	} else if (IN_CLASSC(afsifinfo[i].ipaddr)) {
-	    netmask = IN_CLASSC_NET;
-	} else {
-	    netmask = 0;
-	}
-	net = afsifinfo[i].ipaddr & netmask;
-
-#ifdef notdef
-	if (!s) {
-	    if (afsifinfo[i].ipaddr != 0x7f000001) {	/* ignore loopback */
-		*cnt += 1;
-		if (*cnt > 16)
-		    return;
-		*addrp++ = afsifinfo[i].ipaddr;
-	    }
-	} else
-#endif /* notdef */
-        {
-            /* XXXXXX Do the individual ip ranking below XXXXX */
-            if ((sa->sa_ip & netmask) == net) {
-                if ((sa->sa_ip & subnetmask) == subnet) {
-                    if (afsifinfo[i].ipaddr == sa->sa_ip) {   /* ie, ME!  */
-                        sa->sa_iprank = TOPR;
-                    } else {
-                        sa->sa_iprank = HI + afsifinfo[i].metric; /* case #2 */
-                    }
-                } else {
-                    sa->sa_iprank = MED + afsifinfo[i].metric;    /* case #3 */
-                }
-            } else {
-                    sa->sa_iprank = LO + afsifinfo[i].metric;     /* case #4 */
-            }
-            /* check for case #5 -- point-to-point link */
-            if ((afsifinfo[i].flags & IFF_POINTOPOINT)
-                && (afsifinfo[i].dstaddr == sa->sa_ip)) {
-
-                    if (afsifinfo[i].metric >= (MAXDEFRANK - MED) / PPWEIGHT)
-                        sa->sa_iprank = MAXDEFRANK;
-                    else
-                        sa->sa_iprank = MED + (PPWEIGHT << afsifinfo[i].metric);
-            }
-        }
-    }
-    
-    rw_exit(&afsifinfo_lock);
+    for (ill = ILL_START_WALK_ALL(&ctx) ; ill ; ill = ill_next(&ctx, ill)) {
 #else
     for (ill = (struct ill_s *)*addr /*ill_g_headp */ ; ill;
 	 ill = ill->ill_next) {
+#endif
 #ifdef AFS_SUN58_ENV
 	/* Make sure this is an IPv4 ILL */
 	if (ill->ill_isv6)
@@ -1401,7 +1313,6 @@ static int afs_SetServerPrefs(struct srvAddr *sa) {
 	    }
 	}
     }
-#endif /* AFS_SUN510_ENV */
 #else
 #ifndef USEIFADDR
     struct ifnet *ifn = NULL;
@@ -1472,22 +1383,15 @@ static int afs_SetServerPrefs(struct srvAddr *sa) {
 #elif defined(AFS_DARWIN80_ENV)
     {
 	errno_t t;
-	unsigned int count;
-	int cnt=0, m, j;
-	ifaddr_t *ifads;
-	ifnet_t *ifn;
-
-	if (!ifnet_list_get(AF_INET, &ifn, &count)) {
-	    for (m = 0; m < count; m++) {
-		if (!ifnet_get_address_list(ifn[m], &ifads)) {
-		    for (j = 0; ifads[j] != NULL && cnt < ADDRSPERSITE; j++) {
-			afsi_SetServerIPRank(sa, ifads[j]);
-			cnt++;
-		    }
-		    ifnet_free_address_list(ifads);
-		}
+	int cnt=0;
+	ifaddr_t *addresses, address;
+	t = ifnet_get_address_list_family(NULL, &addresses, AF_INET);
+	if (t == 0) {
+	    while(addresses[cnt] != NULL) {
+		afsi_SetServerIPRank(sa, address);
+		cnt++;
 	    }
-	    ifnet_list_free(ifn);
+	    ifnet_free_address_list(addresses);
 	}
     }
 #elif defined(AFS_DARWIN60_ENV)
@@ -1750,11 +1654,7 @@ struct server *afs_GetServer(afs_uint32 * aserverp, afs_int32 nservers,
 
 	/* Compute preference values and resort */
 	if (!newsa->sa_iprank) {
-	    if (aport == fsport) {
-		afs_SetServerPrefs(newsa);	/* new fileserver rank */
-	    } else {
-		newsa->sa_iprank = 10000 + afs_randomMod127();	/* new vlserver rank */
-	    }
+	    afs_SetServerPrefs(newsa);	/* new server rank */
 	}
     }
     afs_SortOneServer(newts);	/* Sort by rank */
