@@ -58,7 +58,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/afsd/afsd.c,v 1.59 2005/12/01 04:02:23 shadow Exp $");
+    ("$Header: /cvs/openafs/src/afsd/afsd.c,v 1.60 2006/03/23 15:35:53 shadow Exp $");
 
 #define VFS 1
 
@@ -1074,6 +1074,10 @@ CheckCacheBaseDir(char *dir)
 	    return "cannot use reiserfs as cache partition";
 	} else if (statfsbuf.f_type == 0x58465342) {	/* XFS_SUPER_MAGIC */
 	    return "cannot use xfs as cache partition";
+	} else if (statfsbuf.f_type == 0x01021994) {    /* TMPFS_SUPER_MAGIC */
+            return "cannot use tmpfs as cache partition";
+        } else if (statfsbuf.f_type != 0xEF53) {
+            return "must use ext2 or ext3 for cache partition";
 	}
     }
 #endif
@@ -1741,7 +1745,12 @@ mainproc(struct cmd_syndesc *as, char *arock)
 
     if (!(cacheFlags & AFSCALL_INIT_MEMCACHE)
 	&& (fsTypeMsg = CheckCacheBaseDir(cacheBaseDir))) {
+#ifdef AFS_SUN5_ENV
 	printf("%s: WARNING: Cache dir check failed (%s)\n", rn, fsTypeMsg);
+#else
+	printf("%s: ERROR: Cache dir check failed (%s)\n", rn, fsTypeMsg);
+	exit(1);
+#endif
     }
 #if 0
     fputs(AFS_GOVERNMENT_MESSAGE, stdout);
