@@ -14,7 +14,7 @@
 #include <afs/param.h>
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_clock_nt.c,v 1.8 2003/12/07 22:49:36 jaltman Exp $");
+    ("$Header: /cvs/openafs/src/rx/rx_clock_nt.c,v 1.12 2006/03/06 03:04:38 jaltman Exp $");
 
 #ifdef AFS_NT40_ENV
 #include <stdio.h>
@@ -23,6 +23,8 @@ RCSID
 #include <winbase.h>
 #include "rx_clock.h"
 
+void clock_UpdateTime(void);	/* forward reference */
+
 struct clock clock_now;		/* The last elapsed time ready by clock_GetTimer */
 
 /* This is set to 1 whenever the time is read, and reset to 0 whenever
@@ -45,13 +47,11 @@ void
 clock_Init(void)
 {
     if (!QueryPerformanceFrequency(&rxi_clockFreq)) {
-	printf("No High Performance clock, exiting.\n");
+	OutputDebugString("No High Performance clock, exiting.\n");
 	exit(1);
     }
-
     clockInitialized = 1;
-    (void)QueryPerformanceCounter(&rxi_clock0);
-
+    
     clock_UpdateTime();
 }
 
@@ -61,7 +61,7 @@ int
 clock_UnInit(void)
 {
     clockInitialized = 0;
-	return 0;
+    return 0;
 }
 #endif
 
