@@ -15,7 +15,7 @@
 #endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/rx/rx_rdwr.c,v 1.21.2.4 2005/04/14 02:31:44 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rx/rx_rdwr.c,v 1.29 2006/02/28 00:17:52 shadow Exp $");
 
 #ifdef KERNEL
 #ifndef UKERNEL
@@ -28,6 +28,9 @@ RCSID
 #include "h/types.h"
 #include "h/time.h"
 #include "h/stat.h"
+#if defined(AFS_AIX_ENV) || defined(AFS_AUX_ENV) || defined(AFS_SUN5_ENV) 
+#include "h/systm.h"
+#endif
 #ifdef	AFS_OSF_ENV
 #include <net/net_globals.h>
 #endif /* AFS_OSF_ENV */
@@ -68,14 +71,16 @@ RCSID
 #endif /* AFS_OSF_ENV */
 #else /* KERNEL */
 # include <sys/types.h>
-#ifndef AFS_NT40_ENV
+#ifdef AFS_NT40_ENV
+# include <winsock2.h>
+#else /* !AFS_NT40_ENV */
 # include <sys/socket.h>
 # include <sys/file.h>
 # include <netdb.h>
 # include <netinet/in.h>
 # include <sys/stat.h>
 # include <sys/time.h>
-#endif
+#endif /* !AFS_NT40_ENV */
 #ifdef HAVE_STRING_H
 #include <string.h>
 #else
@@ -107,18 +112,15 @@ rxi_ReadProc(register struct rx_call *call, register char *buf,
 {
     register struct rx_packet *cp = call->currentPacket;
     register struct rx_packet *rp;
-    register struct rx_packet *nxp;	/* Next packet pointer, for queue_Scan */
     register int requestCount;
     register unsigned int t;
+
 /* XXXX took out clock_NewTime from here.  Was it needed? */
     requestCount = nbytes;
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
-    if (!queue_IsEmpty(&call->iovq)) {
-	for (queue_Scan(&call->iovq, rp, nxp, rx_packet)) {
-	    queue_Remove(rp);
-	    rxi_FreePacket(rp);
-	}
+    if (queue_IsNotEmpty(&call->iovq)) {
+        rxi_FreePackets(0, &call->iovq);
     }
 
     do {
@@ -310,12 +312,7 @@ rx_ReadProc(struct rx_call *call, char *buf, int nbytes)
      * ReadvProc/WritevProc.
      */
     if (!queue_IsEmpty(&call->iovq)) {
-	register struct rx_packet *rp;
-	register struct rx_packet *nxp;
-	for (queue_Scan(&call->iovq, rp, nxp, rx_packet)) {
-	    queue_Remove(rp);
-	    rxi_FreePacket(rp);
-	}
+        rxi_FreePackets(0, &call->iovq);
     }
 
     /*
@@ -362,12 +359,7 @@ rx_ReadProc32(struct rx_call *call, afs_int32 * value)
      * ReadvProc/WritevProc.
      */
     if (!queue_IsEmpty(&call->iovq)) {
-	register struct rx_packet *rp;
-	register struct rx_packet *nxp;
-	for (queue_Scan(&call->iovq, rp, nxp, rx_packet)) {
-	    queue_Remove(rp);
-	    rxi_FreePacket(rp);
-	}
+	rxi_FreePackets(0, &call->iovq);
     }
 
     /*
@@ -382,14 +374,14 @@ rx_ReadProc32(struct rx_call *call, afs_int32 * value)
     if (!call->error && tcurlen > sizeof(afs_int32)
 	&& tnLeft > sizeof(afs_int32)) {
 	tcurpos = call->curpos;
-	if (!((long)tcurpos & (sizeof(afs_int32) - 1))) {
+	if (!((size_t)tcurpos & (sizeof(afs_int32) - 1))) {
 	    *value = *((afs_int32 *) (tcurpos));
 	} else {
 	    memcpy((char *)value, tcurpos, sizeof(afs_int32));
 	}
 	call->curpos = tcurpos + sizeof(afs_int32);
-	call->curlen = tcurlen - sizeof(afs_int32);
-	call->nLeft = tnLeft - sizeof(afs_int32);
+	call->curlen = tcurlen - (u_short)sizeof(afs_int32);
+	call->nLeft = tnLeft - (u_short)sizeof(afs_int32);
 	return sizeof(afs_int32);
     }
 
@@ -570,7 +562,6 @@ rxi_ReadvProc(struct rx_call *call, struct iovec *iov, int *nio, int maxio,
 	      int nbytes)
 {
     struct rx_packet *rp;
-    struct rx_packet *nxp;	/* Next packet pointer, for queue_Scan */
     int requestCount;
     int nextio;
 
@@ -578,9 +569,8 @@ rxi_ReadvProc(struct rx_call *call, struct iovec *iov, int *nio, int maxio,
     nextio = 0;
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
-    for (queue_Scan(&call->iovq, rp, nxp, rx_packet)) {
-	queue_Remove(rp);
-	rxi_FreePacket(rp);
+    if (queue_IsNotEmpty(&call->iovq)) {
+        rxi_FreePackets(0, &call->iovq);
     }
 
     if (call->mode == RX_MODE_SENDING) {
@@ -657,17 +647,12 @@ rxi_WriteProc(register struct rx_call *call, register char *buf,
 {
     struct rx_connection *conn = call->conn;
     register struct rx_packet *cp = call->currentPacket;
-    register struct rx_packet *tp;	/* Temporary packet pointer */
-    register struct rx_packet *nxp;	/* Next packet pointer, for queue_Scan */
     register unsigned int t;
     int requestCount = nbytes;
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
-    if (!queue_IsEmpty(&call->iovq)) {
-	for (queue_Scan(&call->iovq, tp, nxp, rx_packet)) {
-	    queue_Remove(tp);
-	    rxi_FreePacket(tp);
-	}
+    if (queue_IsNotEmpty(&call->iovq)) {
+	rxi_FreePackets(0, &call->iovq);
     }
 
     if (call->mode != RX_MODE_SENDING) {
@@ -833,13 +818,8 @@ rx_WriteProc(struct rx_call *call, char *buf, int nbytes)
      * RX_CALL_IOVEC_WAIT is always cleared before returning from
      * ReadvProc/WritevProc.
      */
-    if (!queue_IsEmpty(&call->iovq)) {
-	register struct rx_packet *rp;
-	register struct rx_packet *nxp;
-	for (queue_Scan(&call->iovq, rp, nxp, rx_packet)) {
-	    queue_Remove(rp);
-	    rxi_FreePacket(rp);
-	}
+    if (queue_IsNotEmpty(&call->iovq)) {
+	rxi_FreePackets(0, &call->iovq);
     }
 
     /*
@@ -885,13 +865,8 @@ rx_WriteProc32(register struct rx_call *call, register afs_int32 * value)
      * RX_CALL_IOVEC_WAIT is always cleared before returning from
      * ReadvProc/WritevProc.
      */
-    if (!queue_IsEmpty(&call->iovq)) {
-	register struct rx_packet *rp;
-	register struct rx_packet *nxp;
-	for (queue_Scan(&call->iovq, rp, nxp, rx_packet)) {
-	    queue_Remove(rp);
-	    rxi_FreePacket(rp);
-	}
+    if (queue_IsNotEmpty(&call->iovq)) {
+	rxi_FreePackets(0, &call->iovq);
     }
 
     /*
@@ -906,14 +881,14 @@ rx_WriteProc32(register struct rx_call *call, register afs_int32 * value)
     if (!call->error && tcurlen >= sizeof(afs_int32)
 	&& tnFree >= sizeof(afs_int32)) {
 	tcurpos = call->curpos;
-	if (!((long)tcurpos & (sizeof(afs_int32) - 1))) {
+	if (!((size_t)tcurpos & (sizeof(afs_int32) - 1))) {
 	    *((afs_int32 *) (tcurpos)) = *value;
 	} else {
 	    memcpy(tcurpos, (char *)value, sizeof(afs_int32));
 	}
 	call->curpos = tcurpos + sizeof(afs_int32);
-	call->curlen = tcurlen - sizeof(afs_int32);
-	call->nFree = tnFree - sizeof(afs_int32);
+	call->curlen = tcurlen - (u_short)sizeof(afs_int32);
+	call->nFree = tnFree - (u_short)sizeof(afs_int32);
 	return sizeof(afs_int32);
     }
 
@@ -938,8 +913,6 @@ rxi_WritevAlloc(struct rx_call *call, struct iovec *iov, int *nio, int maxio,
 {
     struct rx_connection *conn = call->conn;
     struct rx_packet *cp = call->currentPacket;
-    struct rx_packet *tp;	/* temporary packet pointer */
-    struct rx_packet *nxp;	/* Next packet pointer, for queue_Scan */
     int requestCount;
     int nextio;
     /* Temporary values, real work is done in rxi_WritevProc */
@@ -952,9 +925,8 @@ rxi_WritevAlloc(struct rx_call *call, struct iovec *iov, int *nio, int maxio,
     nextio = 0;
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
-    for (queue_Scan(&call->iovq, tp, nxp, rx_packet)) {
-	queue_Remove(tp);
-	rxi_FreePacket(tp);
+    if (queue_IsNotEmpty(&call->iovq)) {
+        rxi_FreePackets(0, &call->iovq);
     }
 
     if (call->mode != RX_MODE_SENDING) {
@@ -1066,8 +1038,6 @@ int
 rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
 {
     struct rx_packet *cp = call->currentPacket;
-    register struct rx_packet *tp;	/* Temporary packet pointer */
-    register struct rx_packet *nxp;	/* Next packet pointer, for queue_Scan */
     int nextio;
     int requestCount;
     struct rx_queue tmpq;
@@ -1092,14 +1062,11 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
 #endif /* AFS_GLOBAL_RXLOCK_KERNEL */
 
     if (call->error) {
-	for (queue_Scan(&call->iovq, tp, nxp, rx_packet)) {
-	    queue_Remove(tp);
-	    rxi_FreePacket(tp);
-	}
 	if (cp) {
-	    rxi_FreePacket(cp);
+	    queue_Prepend(&call->iovq, cp);
 	    cp = call->currentPacket = NULL;
 	}
+	rxi_FreePackets(0, &call->iovq);
 	return 0;
     }
 
@@ -1126,10 +1093,7 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
 		if (queue_IsEmpty(&call->iovq)) {
 		    call->error = RX_PROTOCOL_ERROR;
 		    cp = call->currentPacket = NULL;
-		    for (queue_Scan(&tmpq, tp, nxp, rx_packet)) {
-			queue_Remove(tp);
-			rxi_FreePacket(tp);
-		    }
+		    rxi_FreePackets(0, &tmpq);
 		    return 0;
 		}
 		cp = queue_First(&call->iovq, rx_packet);
@@ -1150,14 +1114,11 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
 	    if (iov[nextio].iov_base != call->curpos
 		|| iov[nextio].iov_len > (int)call->curlen) {
 		call->error = RX_PROTOCOL_ERROR;
-		for (queue_Scan(&tmpq, tp, nxp, rx_packet)) {
-		    queue_Remove(tp);
-		    rxi_FreePacket(tp);
-		}
 		if (cp) {
-		    rxi_FreePacket(cp);
+		    queue_Prepend(&tmpq, cp);
 		    call->currentPacket = NULL;
 		}
+		rxi_FreePackets(0, &tmpq);
 		return 0;
 	    }
 	    nbytes -= iov[nextio].iov_len;
@@ -1178,10 +1139,7 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
 
     /* Move the packets from the temporary queue onto the transmit queue.
      * We may end up with more than call->twind packets on the queue. */
-    for (queue_Scan(&tmpq, tp, nxp, rx_packet)) {
-	queue_Remove(tp);
-	queue_Append(&call->tq, tp);
-    }
+    queue_SpliceAppend(&call->tq, &tmpq);
 
     if (!(call->flags & (RX_CALL_FAST_RECOVER | RX_CALL_FAST_RECOVER_WAIT))) {
 	rxi_Start(0, call, 0, 0);
@@ -1231,13 +1189,10 @@ void
 rxi_FlushWrite(register struct rx_call *call)
 {
     register struct rx_packet *cp = call->currentPacket;
-    register struct rx_packet *tp;	/* Temporary packet pointer */
-    register struct rx_packet *nxp;	/* Next packet pointer, for queue_Scan */
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
-    for (queue_Scan(&call->iovq, tp, nxp, rx_packet)) {
-	queue_Remove(tp);
-	rxi_FreePacket(tp);
+    if (queue_IsNotEmpty(&call->iovq)) {
+	rxi_FreePackets(0, &call->iovq);
     }
 
     if (call->mode == RX_MODE_SENDING) {
