@@ -19,13 +19,16 @@
 #endif
 
 RCSID
-    ("$Header: /cvs/openafs/src/rxkad/rxkad_client.c,v 1.15 2003/07/15 23:16:42 shadow Exp $");
+    ("$Header: /cvs/openafs/src/rxkad/rxkad_client.c,v 1.22 2006/02/28 00:17:54 shadow Exp $");
 
 #ifdef KERNEL
 #include "afs/stds.h"
 #ifndef UKERNEL
 #include "h/types.h"
 #include "h/time.h"
+#if defined(AFS_AIX_ENV) || defined(AFS_AUX_ENV) || defined(AFS_SUN5_ENV) 
+#include "h/systm.h"
+#endif
 #ifdef AFS_LINUX20_ENV
 #include "h/socket.h"
 #endif
@@ -63,11 +66,10 @@ RCSID
 #endif /* AFS_PTHREAD_ENV */
 #endif /* KERNEL */
 
+#include <des/stats.h>
 #include "private_data.h"
 #define XPRT_RXKAD_CLIENT
 
-char *rxi_Alloc();
-
 #ifndef max
 #define	max(a,b)    ((a) < (b)? (b) : (a))
 #endif /* max */
@@ -103,8 +105,8 @@ static struct rx_securityOps rxkad_client_ops = {
  */
 #include <assert.h>
 pthread_mutex_t rxkad_client_uid_mutex;
-#define LOCK_CUID assert(pthread_mutex_lock(&rxkad_client_uid_mutex)==0);
-#define UNLOCK_CUID assert(pthread_mutex_unlock(&rxkad_client_uid_mutex)==0);
+#define LOCK_CUID assert(pthread_mutex_lock(&rxkad_client_uid_mutex)==0)
+#define UNLOCK_CUID assert(pthread_mutex_unlock(&rxkad_client_uid_mutex)==0)
 #else
 #define LOCK_CUID
 #define UNLOCK_CUID
@@ -121,7 +123,8 @@ rxkad_AllocCID(struct rx_securityClass *aobj, struct rx_connection *aconn)
     struct rxkad_cidgen tgen;
     static afs_int32 counter = 0;	/* not used anymore */
 
-    LOCK_CUID if (Cuid[0] == 0) {
+    LOCK_CUID;
+    if (Cuid[0] == 0) {
 	afs_uint32 xor[2];
 	tgen.ipAddr = rxi_getaddr();	/* comes back in net order */
 	clock_GetTime(&tgen.time);	/* changes time1 and time2 */
@@ -156,12 +159,14 @@ rxkad_AllocCID(struct rx_securityClass *aobj, struct rx_connection *aconn)
     }
 
     if (!aconn) {
-	UNLOCK_CUID return 0;
+	UNLOCK_CUID;
+	return 0;
     }
     aconn->epoch = Cuid[0];
     aconn->cid = Cuid[1];
     Cuid[1] += 1 << RX_CIDSHIFT;
-    UNLOCK_CUID return 0;
+    UNLOCK_CUID;
+    return 0;
 }
 
 /* Allocate a new client security object.  Called with the encryption level,
@@ -191,17 +196,23 @@ rxkad_NewClientSecurityObject(rxkad_level level,
     tcp->type |= rxkad_client;
     tcp->level = level;
     code = fc_keysched(sessionkey, tcp->keysched);
-    if (code)
+    if (code) {
+	rxi_Free(tcp, sizeof(struct rxkad_cprivate));
+	rxi_Free(tsc, sizeof(struct rx_securityClass));
 	return 0;		/* bad key */
+    }
     memcpy((void *)tcp->ivec, (void *)sessionkey, sizeof(tcp->ivec));
     tcp->kvno = kvno;		/* key version number */
     tcp->ticketLen = ticketLen;	/* length of ticket */
-    if (tcp->ticketLen > MAXKTCTICKETLEN)
+    if (tcp->ticketLen > MAXKTCTICKETLEN) {
+	rxi_Free(tcp, sizeof(struct rxkad_cprivate));
+	rxi_Free(tsc, sizeof(struct rx_securityClass));
 	return 0;		/* bad key */
+    }
     memcpy(tcp->ticket, ticket, ticketLen);
 
-    LOCK_RXKAD_STATS rxkad_stats_clientObjects++;
-    UNLOCK_RXKAD_STATS return tsc;
+    INC_RXKAD_STATS(clientObjects);
+    return tsc;
 }
 
 /* client: respond to a challenge packet */
@@ -246,8 +257,8 @@ rxkad_GetResponse(struct rx_securityClass *aobj, struct rx_connection *aconn,
 
     if (level > tcp->level)
 	return RXKADLEVELFAIL;
-    LOCK_RXKAD_STATS rxkad_stats.challenges[rxkad_LevelIndex(tcp->level)]++;
-    UNLOCK_RXKAD_STATS if (v2) {
+    INC_RXKAD_STATS(challenges[rxkad_LevelIndex(tcp->level)]);
+    if (v2) {
 	int i;
 	afs_uint32 xor[2];
 	memset((void *)&r_v2, 0, sizeof(r_v2));
@@ -303,6 +314,8 @@ rxkad_GetResponse(struct rx_securityClass *aobj, struct rx_connection *aconn,
 void
 rxkad_ResetState(void)
 {
-    LOCK_CUID Cuid[0] = 0;
+    LOCK_CUID;
+    Cuid[0] = 0;
     rxkad_EpochWasSet = 0;
-UNLOCK_CUID}
+    UNLOCK_CUID;
+}
