@@ -12,7 +12,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/util/winsock_nt.c,v 1.4 2001/07/12 19:59:23 shadow Exp $");
+RCSID
+    ("$Header: /cvs/openafs/src/util/winsock_nt.c,v 1.6 2005/11/05 06:48:27 jaltman Exp $");
 
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
@@ -24,12 +25,13 @@ RCSID("$Header: /cvs/openafs/src/util/winsock_nt.c,v 1.4 2001/07/12 19:59:23 sha
  * 
  * Returns 0 on success, -1 on error.
  */
-int afs_winsockInit(void)
+int
+afs_winsockInit(void)
 {
     static int once = 1;
 
     if (once) {
-	int code ;
+	int code;
 	WSADATA data;
 	WORD sockVersion;
 
@@ -46,7 +48,9 @@ int afs_winsockInit(void)
     return 0;
 }
 
-int afs_gettimeofday(struct timeval *tv, struct timezone *tz)
+/* This function will begin to fail in the year 2038 */
+int
+afs_gettimeofday(struct timeval *tv, struct timezone *tz)
 {
     struct _timeb myTime;
 
